% qq is the Hessian, vv is the vector of linear coefficients
%
qq = [ 1, 0; 0, 30 ];
vv = [ 0; 0 ];

% choose the quadratic objective. this notation defines a "closure", returning
% a function which takes xx as its only parameter, and calls
% objective_quadratic with parameters qq and vv defined above, and xx (the
% parameter of the closure)
%
ff = @( xx ) objective_quadratic( qq, vv, xx );

% use "exact" line search. this defines a closure which takes three parameters.
% note that the epsilon parameter to the exact line search is 1e-4
%
line_search = @( ff, xx, direction ) line_search_bisection( ff, xx, direction, 1e-8 );

% start at (4,0.3), with (for BFGS) an identity inverse Hessian
%
initial_xx = [ 4; 0.3 ];
initial_inverse_hessian = eye( 2 );

% run the algorithms
%
[ xx, values, runtimes, gd_xxs ] = algorithm_gd( ff, initial_xx, line_search, 1e-4 );
gd_iterations = length( values ) - 1;
[ xx, values, runtimes, cg_xxs ] = algorithm_cg( ff, initial_xx, line_search, 1e-4 );
cg_iterations = length( values ) - 1;
[ xx, values, runtimes, bfgs_xxs ] = algorithm_bfgs( ff, initial_xx, initial_inverse_hessian, line_search, 1e-4 );
bfgs_iterations = length( values ) - 1;
[ xx, values, runtimes, newton_xxs ] = algorithm_newton( ff, initial_xx, line_search, 1e-4 );
newton_iterations = length( values ) - 1;


% create the plots

[ XX, YY ] = meshgrid( -5:0.1:5, -5:0.1:5 );
ZZ = qq( 1, 1 ) * ( XX .* XX ) + ( qq( 1, 2 ) + qq( 2, 1 ) ) * ( XX .* YY ) + qq( 2, 2 ) * ( YY .* YY ) + vv( 1 ) * XX + vv( 2 ) * YY;
lower = sqrt( min( min( ZZ ) ) );
upper = sqrt( max( max( ZZ ) ) );

contour( XX, YY, ZZ, ( ( 0.05:0.1:0.95 ) * ( upper - lower ) + lower ) .^ 2 );
colormap( gray )

for ii = 1:(max([gd_iterations,cg_iterations,bfgs_iterations,newton_iterations])+1)

    handle_gd = line( gd_xxs( 1, 1:( min(ii, gd_iterations+1) ) ), gd_xxs( 2, 1:( min(ii, gd_iterations+1) ) ), 'LineWidth', 1.2, 'Color', 'red', 'Marker', 'o' );    
    
    handle_cg = line( cg_xxs( 1, 1:( min(ii, cg_iterations+1) ) ), cg_xxs( 2, 1:( min(ii, cg_iterations+1) ) ), 'LineWidth', 1.2, 'Color', 'green', 'Marker', 'o' );
    
    handle_bfgs = line( bfgs_xxs( 1, 1:( min(ii, bfgs_iterations+1) ) ), bfgs_xxs( 2, 1:( min(ii, bfgs_iterations+1) ) ), 'LineWidth', 1.2, 'Color', 'blue', 'Marker', 'o' );
    
    handle_newton = line( newton_xxs( 1, 1:( min(ii, newton_iterations+1) ) ), newton_xxs( 2, 1:( min(ii, newton_iterations+1) ) ), 'LineWidth', 1.2, 'Color', 'magenta', 'Marker', 'o' );

    legend( [ handle_gd, handle_cg, handle_bfgs, handle_newton ], sprintf( 'GD, %d iterations', min(ii-1,gd_iterations) ), sprintf( 'CG, %d iterations', min(ii-1,cg_iterations) ), sprintf( 'BFGS, %d iterations', min(ii-1,bfgs_iterations) ), sprintf( 'Newton, %d iterations', min(ii-1,newton_iterations) ) );
    if(ii < 6)
        disp( 'Press any key to continue' );
        pause;
    else
        drawnow;
    end

    
end

