% aa is the matrix of coefficients, and bb is the bias. these parameters
% are taken from equation 9.20 of Boyd and Vandenberghe (pg 470)
%
aa = [ 1, 3; 1, -3; -1, 0 ];
bb = [ -0.1; -0.1; -0.1 ];

% choose the sum-exp objective. this notation defines a "closure", returning a
% function which takes xx as its only parameter, and calls objective_sum_exp
% with parameters aa and bb defined above, and xx (the parameter of the
% closure)
%
ff = @( xx ) objective_sum_exp( aa, bb, xx );

% use backtracking line search. this defines a closure which takes three
% parameters.  note that the alpha and beta parameters to the backtracking line
% search are 0.4 and 0.9, respectively
%
line_search = @( ff, xx, direction ) line_search_backtracking( ff, xx, direction, 0.4, 0.9 );

% start at (-1,1.5), with (for BFGS) an identity inverse Hessian
%
initial_xx = [ -1; 1.5 ];
initial_inverse_hessian = eye( 2 );

% run the algorithms
%
[ xx, values, runtimes, gd_xxs ] = algorithm_gd( ff, initial_xx, line_search, 1e-4 );
gd_iterations = length( values ) - 1;
[ xx, values, runtimes, cg_xxs ] = algorithm_cg( ff, initial_xx, line_search, 1e-4 );
cg_iterations = length( values ) - 1;
[ xx, values, runtimes, bfgs_xxs ] = algorithm_bfgs( ff, initial_xx, initial_inverse_hessian, line_search, 1e-4 );
bfgs_iterations = length( values ) - 1;
[ xx, values, runtimes, newton_xxs ] = algorithm_newton( ff, initial_xx, line_search, 1e-4 );
newton_iterations = length( values ) - 1;


% create the plots

[ XX, YY ] = meshgrid( -3:0.04:1, -2:0.04:2 );
ZZ = zeros( size( XX ) );
for ii = 1:size( aa, 1 )
	ZZ = ZZ + exp( aa( ii, 1 ) * XX + aa( ii, 2 ) * YY + bb( ii ) );
end
lower = log( min( min( ZZ ) ) );
upper = log( max( max( ZZ ) ) );

contour( XX, YY, ZZ, exp( ( 0.05:0.1:0.95 ) * ( upper - lower ) + lower ) );
colormap( gray )


for ii = 1:(max([gd_iterations,cg_iterations,bfgs_iterations,newton_iterations])+1)

    handle_gd = line( gd_xxs( 1, 1:( min(ii, gd_iterations+1) ) ), gd_xxs( 2, 1:( min(ii, gd_iterations+1) ) ), 'LineWidth', 1.2, 'Color', 'red', 'Marker', 'o' );    
    
    handle_cg = line( cg_xxs( 1, 1:( min(ii, cg_iterations+1) ) ), cg_xxs( 2, 1:( min(ii, cg_iterations+1) ) ), 'LineWidth', 1.2, 'Color', 'green', 'Marker', 'o' );
    
    handle_bfgs = line( bfgs_xxs( 1, 1:( min(ii, bfgs_iterations+1) ) ), bfgs_xxs( 2, 1:( min(ii, bfgs_iterations+1) ) ), 'LineWidth', 1.2, 'Color', 'blue', 'Marker', 'o' );
    
    handle_newton = line( newton_xxs( 1, 1:( min(ii, newton_iterations+1) ) ), newton_xxs( 2, 1:( min(ii, newton_iterations+1) ) ), 'LineWidth', 1.2, 'Color', 'magenta', 'Marker', 'o' );

    legend( [ handle_gd, handle_cg, handle_bfgs, handle_newton ], sprintf( 'GD, %d iterations', min(ii-1,gd_iterations) ), sprintf( 'CG, %d iterations', min(ii-1,cg_iterations) ), sprintf( 'BFGS, %d iterations', min(ii-1,bfgs_iterations) ), sprintf( 'Newton, %d iterations', min(ii-1,newton_iterations) ) );
    if(ii < 20)
        disp( 'Press any key to continue' );
        pause;
    else
        drawnow;
    end

    
end
